using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using pdftron;
using pdftron.PDF;
using pdftron.Common;

namespace PDFViewCS
{
	/// <summary>
	/// MyPDFView is a sample class derived from pdftron.PDF.PDFViewCtrl 
	/// that illustrates how to implement various custom tools and custom 
	/// drawing on top of PDFViewCtrl. If you only need to view PDF documents 
	/// you can completely comment-out this class and instantiate 
	/// pdftron.PDF.PDFViewCtrl directly (as shown in PDFViewSimple sample project).
	/// 
	// PDFViewCtrl offers built-in support for a number of PDF tools (including markup, 
	// link creation, forms, text selection, etc) and user interface elements (such as 
	// navigation panel, find text, etc). PDFViewCtrl is highly configurable allowing 
	// users to implement custom tools or to modify behavior of built-in tools. 
	// Default user interface elements such as the navigation panel can be replaced 
	// with a custom implementation and users can override window events and various 
	// document actions. For example, by default, PDFViewCtrl will launch URL links 
	// in a web browser and will jump to a destination for 'GoTo' actions. The default 
	// behavior can be overridden using a custom callback function (as shown in the 
	// following sample; see OnAction override).
	/// 
	/// MyPDFView shows how to implement various custom PDF tools such as a 
	/// tool to create new link annotations, a rectangular zoom, a freehand draw,
	/// link navigation, etc).
	/// </summary>
	public class MyPDFView : PDFViewCtrl
	{
		public enum CustomToolMode 
		{
			e_none,

			// Custom Markup tool
			e_custom_draw,                

			// Custom Link-create tool
			e_custom_link_create,         

			// Custom Rectangular zoom-in tool
			e_custom_zoomin,              

			// Custom Rectangular zoom-out tool
			e_custom_zoomout,             

			e_custom_link_action,

			e_custom_mouse_hover, 

			// Extends the built-in text selection tool to implement a new tool
			// used to create hyper-links.
			e_custom_text_select_tool

			// <-- Add new custom tools 
		}

		public static ToolMode _base_tool_mode = ToolMode.e_pan;
		public static CustomToolMode _tool_mode = CustomToolMode.e_none;

		public MyPDFView(PDFViewForm parent) : base() 
		{			
			_parent = parent;			
			_popup.Popup += new EventHandler(PopupEventHandler);
			this.ContextMenu = _popup;

			// Customize PDFViewCtrl ...
			
			// this.ForeColor = Color.White;
			// this.BackColor = Color.White;
			// this.SetPageBorderVisibility(false);

			// Use built-in navigation panel?
			// In case you would like to implement your own document navigation, 
			// disable the  built-in navigation panel and add a custom panel along
			// the lines 'CUSTOM_NAV' code region in 'PDFViewForm.cs'.
			ShowNavToolbar(true);  

			// By default PDFNet will use document preferences PDFDocViewPrefs
			// to display navigation panel. To programatically override the initial 
			// panel visibility, call _pdfview.ShowNavPanel(visible) immediately after
			// associating document with the view in 'PDFViewForm.cs'.
			
			// SetEnabledPanels((int)PDFViewCtrl.PanelType.e_bookmarks+(int)PDFViewCtrl.PanelType.e_thumbview); // Sets which navigation buttons will appear in the nvigation toolbar.
			// EnableScrollbar(false);

			// SetProgressiveRendering(false);
			// SetDrawAnnotations(false);
			// ...
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			if ((_tool_mode == CustomToolMode.e_custom_link_create ||
				 _tool_mode == CustomToolMode.e_custom_zoomin || 
				 _tool_mode == CustomToolMode.e_custom_zoomout)
				&& Capture && _cur_page>0) 
			{
				// Draw Link rectangle annotations on top of the PDF page.
				float top, bottom, left, right;
				if (_start_pt.Y < _end_pt.Y) 
				{
					top = _start_pt.Y;
					bottom = _end_pt.Y;
				}
				else 
				{
					bottom = _start_pt.Y;
					top = _end_pt.Y;
				}

				if (_start_pt.X < _end_pt.X) 
				{
					left = _start_pt.X;
					right = _end_pt.X;
				}
				else 
				{
					right = _start_pt.X;
					left = _end_pt.X;
				}
				
				e.Graphics.DrawRectangle(Pens.Red, left, top, right-left, bottom-top);
			}
		
			if (_freehand_markup != null) // Draw FreeHand annotations on top.
			{   
				PagePresentationMode pmode = GetPagePresentationMode();
				int curpage = GetCurrentPage();
				
				Graphics g = e.Graphics;
				foreach( DictionaryEntry itr in _freehand_markup)
				{
					//System.Threading.Thread.Sleep(50);

					int annot_page_num = (int) itr.Key;
					if ((pmode == PDFViewCtrl.PagePresentationMode.e_single_page && annot_page_num!=curpage) || 
						(pmode == PDFViewCtrl.PagePresentationMode.e_facing && annot_page_num!=curpage && annot_page_num!=curpage+1))
					{
						continue;
					}

					GraphicsState gs = g.Save();
					Matrix page2scr_mtx = GetDeviceTransform(annot_page_num);
					g.MultiplyTransform(page2scr_mtx);
					FreeHandAnnot annot = (FreeHandAnnot) itr.Value;
					annot.DrawAnnots(g, Pens.Red);
					g.Restore(gs);
				}
			}
		}

		// By default, PDFViewCtrl will launch URL links in a web browser and 
		// will jump to a destination for 'GoTo' actions. The default behavior can 
		// be overloaded using OnAction method.
		public override Boolean OnAction(pdftron.PDF.Action action)
		{
			if (action.IsValid() == false) return true;

			pdftron.PDF.Action.Type type = action.GetType();
			if (type == pdftron.PDF.Action.Type.e_GoTo) 
			{
				Destination dest = action.GetDest();
				if (!dest.IsValid()) 
				{
					MessageBox.Show("Destination is not valid", "Custom PDF Action Handler");
				}
				else 
				{
					MessageBox.Show("  Links to: page number " + dest.GetPage().GetIndex().ToString() + " in this document", "Custom PDF Action Handler");
				}
			}
			else if (type == pdftron.PDF.Action.Type.e_URI) 
			{
				MessageBox.Show("  Links to: " + action.GetSDFObj().Get("URI").Value().GetAsPDFText(), "Custom PDF Action Handler");
			}
			else  
			{
				MessageBox.Show("Other Action Type ", "Custom PDF Action Handler");
			}

			// If this function returns true, PDFViewCtrl will not execute the default 
			// action internally (in case of URL the default action is to open the URL 
			// link in a web browser).
			return false; // -> execute the default action.
		}

		protected override void OnPaintBackground(PaintEventArgs pevent) {}


		protected PointF _start_pt, _end_pt;
		protected int _cur_page = 0;

		// A dictionary mapping page numbers to FreeHandAnnot-s (used to implement freehand markup tool)
		public Hashtable _freehand_markup = null;
		protected GraphicsPath _curstroke = null;  // Used to track the current stroke.
		
		protected override void OnMouseDown(MouseEventArgs e) 
		{
			int splp = base.GetSplitPosition();
			Rect r = base.GetChildWindowPosition();

			base.OnMouseDown(e);  // First process the event in the base class

			if (_tool_mode == CustomToolMode.e_custom_link_create ||
				_tool_mode == CustomToolMode.e_custom_zoomin || 
				_tool_mode == CustomToolMode.e_custom_zoomout) 
			{
				_cur_page = GetPageNumberFromScreenPt(e.X, e.Y);
				if (_cur_page < 1) 
				{
					_cur_page = GetCurrentPage();
				}

				// Record the position for the rectangle origin.
				_start_pt.X = e.X; _start_pt.Y = e.Y;
				Capture = true;
			}
			else if (_tool_mode == CustomToolMode.e_custom_draw) 
			{
				// Sample code illustrating how to draw new content on top 
				// of PDF page...
				_cur_page = GetPageNumberFromScreenPt(e.X, e.Y);
				if (_cur_page < 1) 
				{
					_cur_page = GetCurrentPage();
				}

				if (_freehand_markup == null) _freehand_markup = new Hashtable();

				FreeHandAnnot annot;
				if (!_freehand_markup.ContainsKey(_cur_page))	
				{
					annot = new FreeHandAnnot();
					_freehand_markup.Add(_cur_page, annot);
				}
				else 
				{
					annot = (FreeHandAnnot) _freehand_markup[_cur_page];
				}
				
				_curstroke = new GraphicsPath();  // Start a new stroke.
				annot.AddPath(_curstroke);

				// Record the current position.
				double x=e.X, y=e.Y;
				this.ConvScreenPtToPagePt(ref x, ref y, _cur_page);
				_start_pt.X = (float)x; _start_pt.Y = (float)y;
				Capture = true;
			}
			else if (_tool_mode == CustomToolMode.e_custom_link_action)
			{
				// Sample code illustrating how to implement link navigation.
				if (IsFinishedRendering())
				{
					int page_num = GetPageNumberFromScreenPt(e.X, e.Y);
					if (page_num<1) return;

					// Find the click point in page coordinate system...
					double x = e.X, y = e.Y;
					ConvScreenPtToPagePt(ref x, ref y, page_num);

					Page page = GetDoc().GetPage(page_num);
					int annot_num = page.GetNumAnnots();
					for (int i=0; i<annot_num; ++i) 
					{
						Annot annot = page.GetAnnot(i);
						// Process only link annotations...
						if (annot.IsValid() == false || 
							annot.GetType() != Annot.Type.e_Link) continue;

						Rect box = annot.GetRect();
						if (box.Contains(x, y))
						{
							// Execute the action - jump to the destination page 
							pdftron.PDF.Action action = new pdftron.PDF.Annots.Link(annot).GetAction(); 
							if (action.IsValid() == false) continue;
							if (action.GetType() == pdftron.PDF.Action.Type.e_GoTo) 
							{
								Destination dest = action.GetDest();
								if (dest.IsValid())
								{
									page_num = dest.GetPage().GetIndex(); 
									SetCurrentPage(page_num);
									return;
								}
							}
							else if (action.GetType() == pdftron.PDF.Action.Type.e_URI) 
							{
								// open the browser and jump to the given hyperlink...
								string target;
								
								target=action.GetSDFObj().Get("URI").Value().GetAsPDFText();
								try
								{
									System.Diagnostics.Process.Start(target);
								}
								catch(System.ComponentModel.Win32Exception noBrowser) 
								{
									if (noBrowser.ErrorCode==-2147467259)
										MessageBox.Show(noBrowser.Message);
								}
								catch (System.Exception other)
								{
									MessageBox.Show(other.Message);
								}
								break;
							}
							else if (action.GetType() == pdftron.PDF.Action.Type.e_GoToR) 
							{
								// if the remote document is PDF, possibly open the
								// document as a new PDFViewForm...
							}
						}					
					}
				}
			}
		}

		protected override void OnMouseMove(MouseEventArgs e) 
		{
			base.OnMouseMove(e);  // First process the event in the base class

			if ((_tool_mode == CustomToolMode.e_custom_link_create ||
				 _tool_mode == CustomToolMode.e_custom_zoomin || 
				 _tool_mode == CustomToolMode.e_custom_zoomout)		
				&& Capture && _cur_page>0) 
			{
				_end_pt.X = e.X; _end_pt.Y = e.Y;
				Invalidate();
			}
			else if (_tool_mode == CustomToolMode.e_custom_mouse_hover)
			{   
				// Sample code showing how to change mouse cursor when hovering
				// over link annotations (in pan mode)...
				if (IsFinishedRendering())
				{
					int page_num = GetPageNumberFromScreenPt(e.X, e.Y);
					if (page_num<1) return;

					// Find the point in PDF page coordinate system...
					double x = e.X, y = e.Y;
					ConvScreenPtToPagePt(ref x, ref y, page_num);

					Page page = GetDoc().GetPage(page_num);
					int annot_num = page.GetNumAnnots();
					bool over_link = false;
					for (int i=0; i<annot_num; ++i)  
					{
						Annot annot = page.GetAnnot(i);
						if (annot.IsValid() == false || 
							annot.GetType() != Annot.Type.e_Link) continue;

						Rect box = annot.GetRect();
						if (box.Contains(x, y)) 
						{
							over_link = true;
							break;
						}
					}

					if (over_link) Cursor = Cursors.Cross;
					else Cursor = Cursors.Hand;
				}
			}
			else if (_tool_mode == CustomToolMode.e_custom_draw && Capture && _curstroke!=null)
			{
				double x=e.X, y=e.Y;
				this.ConvScreenPtToPagePt(ref x, ref y, _cur_page);
				_end_pt.X = (float)x; _end_pt.Y = (float)y;
				_curstroke.AddLine(_start_pt.Y, _start_pt.X, _end_pt.Y, _end_pt.X);
				_start_pt.X = _end_pt.X; _start_pt.Y = _end_pt.Y;
				Invalidate();
			}
		}	

		protected override void OnMouseUp(MouseEventArgs e) 
		{
			//// One way to override built-in content menu. Replace it with your own menu?
			//if (e.Button == MouseButtons.Right) 
			//{
			//	MenuItem menuItem = new MenuItem("Print");
			//	menuItem.Click += new EventHandler(OnPrint);
			//	ContextMenu popup = new ContextMenu();
			//	popup.MenuItems.Add(menuItem);
			//	Rect wnd_pos = GetChildWindowPosition();
			//	popup.Show(this, new System.Drawing.Point(e.X+(int)wnd_pos.x1, e.Y+(int)wnd_pos.y1));
			//	return;
			//}

			base.OnMouseUp(e);  // First process the event in the base class

			if (_tool_mode == CustomToolMode.e_custom_link_create && Capture && _cur_page>0) 
			{
				// The following sample code illustrates an implementation of link annotation 
				// tool that can be used to create new links on top of existing PDF documents.
				PDFDoc doc = GetDoc();
				Page pg = doc.GetPage(_cur_page);
				if (pg != null) 
				{
					_end_pt.X = e.X; _end_pt.Y = e.Y;
					pdftron.PDF.Action action = CreateLinkHelper();
					if (action != null) 
					{
						// Create link annotation position rectangle in PDF page coordinate system.
						PointF[] pt_arr = { _start_pt, _end_pt };
						Matrix screen2page = GetDeviceTransform(_cur_page);
						screen2page.Invert();
						screen2page.TransformPoints(pt_arr);

						Rect pos = new Rect(pt_arr[0].Y, pt_arr[0].X, pt_arr[1].Y, pt_arr[1].X);
						pos.Normalize();

						pdftron.PDF.Annots.Link link = pdftron.PDF.Annots.Link.Create(GetDoc(), pos, action);
						link.SetBorderStyle(new Annot.BorderStyle(Annot.BorderStyle.Style.e_solid, 1));
						link.SetColor(new ColorPt(1, 0, 0));
						pg.AnnotPushBack(link);
					}
				}

				Capture = false;
				_cur_page = 0;
				Invalidate();
			}
			else if ((_tool_mode == CustomToolMode.e_custom_zoomin  || 
					  _tool_mode == CustomToolMode.e_custom_zoomout) 
				&& Capture && _cur_page>0) 
			{
				// The following sample code illustrates an implementation of rectangular 
				// zoom-in and zoom-out tool (similar to the one in Acrobat Reader).

				_end_pt.X = e.X; _end_pt.Y = e.Y;
				double zoom_width = _end_pt.X - _start_pt.X; 
				double zoom_height = _end_pt.Y - _start_pt.Y; 

				if (Math.Abs(zoom_width)>20 && Math.Abs(zoom_height)>20) 
				{
					double z1, z2;
					if (_tool_mode == CustomToolMode.e_custom_zoomin) 
					{
						z1 = Width/Math.Abs(zoom_width); 
						z2 = Height/Math.Abs(zoom_height); 
					}
					else  
					{
						z1 = Math.Abs(zoom_width)/Width;
						z2 = Math.Abs(zoom_height)/Height; 
					}	
				
					double z = Math.Min(z1, z2) * GetZoom();
					SetZoom((int)(_start_pt.X + zoom_width/2.0), 
						(int)(_start_pt.Y + zoom_height/2.0), z);
				}

				Capture = false;
				_cur_page = 0;
				Invalidate();
			}
			else if (_tool_mode == CustomToolMode.e_custom_draw && Capture)
			{   // Freehand tool
				double x=e.X, y=e.Y;
				this.ConvScreenPtToPagePt(ref x, ref y, _cur_page);
				_curstroke.AddLine(_start_pt.Y, _start_pt.X, (float)y, (float)x);

				Invalidate();
				_curstroke = null;
				Capture = false;
				Invalidate();
			}
			else if (_tool_mode == CustomToolMode.e_custom_text_select_tool && HasSelection()) 
			{
				// Extends the built-in text selection tool to implement a new tool
				// used to create hyper-links.
				pdftron.PDF.Action action = CreateLinkHelper();
				if (action != null) 
				{
					Selection s = GetSelection();
					Double[] pt_arr = s.GetQuads();
					int num_rects = pt_arr.Length/8;
					Page pg = GetDoc().GetPage(s.GetPageNum());
					for (int i=0; i<num_rects; ++i) 
					{
						pdftron.PDF.Annots.Link link = pdftron.PDF.Annots.Link.Create(GetDoc(), new Rect(pt_arr[i*8], pt_arr[i*8+1], pt_arr[i*8+4], pt_arr[i*8+5]), action);
						// Set the link appearance (optional)
						// link.SetBorderStyle(new Annot.BorderStyle(Annot.BorderStyle.Style.e_solid, 1));
						// link.SetColor(new ColorPt(1, 0, 0));
						pg.AnnotPushBack(link);
					}
				}

				Update();  // Refresh the view only if links are supposed to be visible.
			}
		}

		/*
		// Override built-in double click event?
		protected override void OnDoubleClick(EventArgs e) 
		{
			MessageBox.Show("Custom OnDoubleClick handler", "Test", MessageBoxButtons.OK);
		}	 
				 
		// Override built-in mouse wheel event handling?
		protected override void OnMouseWheel(MouseEventArgs e)
		{
			if (e.Delta > 0)
				GotoPreviousPage();
			else if (e.Delta < 0)
				GotoNextPage();
		}*/ 

		private ContextMenu _popup = new ContextMenu();

		private void PopupEventHandler(System.Object sender, System.EventArgs e)
		{			
			_popup.MenuItems.Clear();	// Clear all previously added MenuItems.

			if(HasSelection()) // Add MenuItems to display if there is a text selection.
			{				
				MenuItem menuItem1 = new MenuItem("Cop&y");
				menuItem1.Click += new EventHandler(OnTextCopy);
				_popup.MenuItems.Add(menuItem1);
			}

			if (_tool_mode == CustomToolMode.e_custom_mouse_hover) 
			{
				MenuItem menuItem2 = new MenuItem("Se&lect All");
				menuItem2.Click += new EventHandler(OnSelectAll);
				_popup.MenuItems.Add(menuItem2);

				MenuItem menuItem3 = new MenuItem("D&eselect All");
				menuItem3.Click += new EventHandler(OnDeselectAll);
				_popup.MenuItems.Add(menuItem3);

			}
					
			{   // Context menu for all tool modes.
				//MenuItem menuItem1 = new MenuItem("Hand Tool");
				//menuItem1.Click += new EventHandler(OnSelectHandTool);
				//_popup.MenuItems.Add(menuItem1);

				//MenuItem menuItem2 = new MenuItem("Text Select Tool");
				//menuItem2.Click += new EventHandler(OnSelectTextTool);
				//_popup.MenuItems.Add(menuItem2);

				MenuItem menuItem3 = new MenuItem("Print");
				menuItem3.Click += new EventHandler(OnPrint);
				_popup.MenuItems.Add(menuItem3);
			}
		}

		public void OnTextCopy(object sender, EventArgs e)
		{
			Selection sel = GetSelection();
			// NET2 method: Clipboard.SetText(sel.GetText(), TextDataFormat.Html);
			// doesn't work in some applications, so we provide our own implementation:
			MyPDFView.CopyHtmlToClipBoard(sel.GetAsHtml(), sel.GetAsUnicode());
		}

		private void OnSelectAll(object sender, EventArgs e) {
			SelectAll();
		}

		private void OnDeselectAll(object sender, EventArgs e) {
			ClearSelection();
		}

		// private void OnSelectTextTool(object sender, EventArgs e) { _parent.SetToolMode(CustomToolMode.e_text_rect_select); }
		// private void OnSelectHandTool(object sender, EventArgs e) {	_parent.SetToolMode(CustomToolMode.e_pan); }

		private void OnPrint(object sender, EventArgs e) 
		{
			_parent.Print();
		}

		private PDFViewForm _parent;

		// A utility method to copy HTML selection to clipboard.
		static public void CopyHtmlToClipBoard(string html, string txt) 
		{
			System.Text.Encoding enc = System.Text.Encoding.UTF8;

			string begin = "Version:0.9\r\nStartHTML:{0:000000}\r\nEndHTML:{1:000000}"
				+ "\r\nStartFragment:{2:000000}\r\nEndFragment:{3:000000}\r\n";

			string html_begin = "<html>\r\n<head>\r\n"
				+ "<meta http-equiv=\"Content-Type\""
				+ " content=\"text/html; charset=" + enc.WebName + "\">\r\n"
				+ "<title>HTML clipboard</title>\r\n</head>\r\n<body>\r\n"
				+ "<!--StartFragment-->";

			string html_end = "<!--EndFragment-->\r\n</body>\r\n</html>\r\n";

			string begin_sample = String.Format(begin, 0, 0, 0, 0);

			int count_begin = enc.GetByteCount(begin_sample);
			int count_html_begin = enc.GetByteCount(html_begin);
			int count_html = enc.GetByteCount(html);
			int count_html_end = enc.GetByteCount(html_end);

			string html_total = String.Format(
				begin
				, count_begin
				, count_begin + count_html_begin + count_html + count_html_end
				, count_begin + count_html_begin
				, count_begin + count_html_begin + count_html
				) + html_begin + html + html_end;

			DataObject obj = new DataObject();
			obj.SetData(DataFormats.Html, new System.IO.MemoryStream(
				enc.GetBytes(html_total)));

			obj.SetData(DataFormats.UnicodeText, txt);

			Clipboard.SetDataObject(obj, true);
		}

		private pdftron.PDF.Action CreateLinkHelper()
		{
			LinkCreate dlg = new LinkCreate();
			if(dlg.ShowDialog() == DialogResult.OK) 
			{
				if (dlg.link.Text == "")
				{
					MessageBox.Show("The link text was not entered.", "PDFViewCtrl", MessageBoxButtons.OK);
				}
				else 
				{
					pdftron.PDF.Action action = null;
					if (dlg.goto_webpage.Checked) 
					{
						if (dlg.link.Text.StartsWith("http://"))
						{
							// Create a URI action
							pdftron.SDF.Obj a = GetDoc().CreateIndirectDict();  
							a.PutName("S", "URI");
							a.PutString("URI", dlg.link.Text);
							action = new pdftron.PDF.Action(a);
						}
						else 
						{
							MessageBox.Show("The hyperlink must start with 'http://'", "PDFViewCtrl", MessageBoxButtons.OK);
						}
					}
					else 
					{
						try 
						{
							int page_num = System.Convert.ToInt32(dlg.link.Text);
							if (page_num < 1 || page_num>GetDoc().GetPageCount()) 
							{
								MessageBox.Show("Invalid page number.", "PDFViewCtrl", MessageBoxButtons.OK);
							}
							else 
							{
								Page dest_page = GetDoc().GetPage(page_num);
								action = pdftron.PDF.Action.CreateGoto(Destination.CreateFitH(dest_page, dest_page.GetPageHeight()));
							}
						}
						catch (Exception ) 
						{
							MessageBox.Show("Invalid page number.", "PDFViewCtrl", MessageBoxButtons.OK);
						}
					}

					return action;
				}
			}

			return null;
		}
	}
}
